@@ -10,7 +10,8 @@ LICENSE END */
 
 use core::convert::TryInto;
 use sixtyfps_compilerlib::langtype::Type as LangType;
-use sixtyfps_corelib::{Brush, ImageReference, PathData, SharedString, SharedVector};
+use sixtyfps_corelib::graphics::Image;
+use sixtyfps_corelib::{Brush, PathData, SharedString, SharedVector};
 use std::collections::HashMap;
 use std::iter::FromIterator;
 use std::path::{Path, PathBuf};
@@ -21,6 +22,8 @@ pub use sixtyfps_compilerlib::diagnostics::{Diagnostic, DiagnosticLevel};
 
 /// This enum represents the different public variants of the [`Value`] enum, without
 /// the contained values.
+// NOTE: The docs for ValueType are duplicated in sixtyfps_interpreter.h, for extraction by
+// Doxygen. Keep in sync!
 #[derive(Debug, Copy, Clone, PartialEq)]
 #[repr(i8)]
 pub enum ValueType {
@@ -34,12 +37,14 @@ pub enum ValueType {
     Bool,
     /// An Array in the .60 language.
     Array,
-    /// A more complex model which is not created by the interpreter itself (Value::Array can also be used for model)
+    /// A more complex model which is not created by the interpreter itself (ValueType::Array can also be used for models)
     Model,
     /// An object
     Struct,
     /// Correspond to `brush` or `color` type in .60.  For color, this is then a [`Brush::SolidColor`]
     Brush,
+    /// Correspond to `image` type in .60.
+    Image,
     /// The type is not a public type but something internal.
     Other = -1,
 }
@@ -92,10 +97,10 @@ pub enum Value {
     Bool(bool),
     #[doc(hidden)]
     /// Correspond to the `image` type in .60
-    Image(ImageReference),
+    Image(Image),
     /// An Array in the .60 language.
     Array(SharedVector<Value>),
-    /// A more complex model which is not created by the interpreter itself (Value::Array can also be used for model)
+    /// A more complex model which is not created by the interpreter itself (Value::Array can also be used for models)
     Model(Rc<dyn sixtyfps_corelib::model::Model<Data = Value>>),
     /// An object
     Struct(Struct),
@@ -108,9 +113,11 @@ pub enum Value {
     /// An easing curve
     EasingCurve(sixtyfps_corelib::animations::EasingCurve),
     #[doc(hidden)]
-    /// An enumation, like `TextHorizontalAlignment::align_center`, represented by `("TextHorizontalAlignment", "align_center")`.
+    /// An enumeration, like `TextHorizontalAlignment::align_center`, represented by `("TextHorizontalAlignment", "align_center")`.
     /// FIXME: consider representing that with a number?
     EnumerationValue(String, String),
+    #[doc(hidden)]
+    LayoutCache(SharedVector<f32>),
 }
 
 impl Value {
@@ -153,6 +160,7 @@ impl PartialEq for Value {
             Value::EnumerationValue(lhs_name, lhs_value) => {
                 matches!(other, Value::EnumerationValue(rhs_name, rhs_value) if lhs_name == rhs_name && lhs_value == rhs_value)
             }
+            Value::LayoutCache(lhs) => matches!(other, Value::LayoutCache(rhs) if lhs == rhs),
         }
     }
 }
@@ -172,6 +180,7 @@ impl std::fmt::Debug for Value {
             Value::PathElements(e) => write!(f, "Value::PathElements({:?})", e),
             Value::EasingCurve(c) => write!(f, "Value::EasingCurve({:?})", c),
             Value::EnumerationValue(n, v) => write!(f, "Value::EnumerationValue({:?}, {:?})", n, v),
+            Value::LayoutCache(v) => write!(f, "Value::LayoutCache({:?})", v),
         }
     }
 }
@@ -208,11 +217,12 @@ macro_rules! declare_value_conversion {
 declare_value_conversion!(Number => [u32, u64, i32, i64, f32, f64, usize, isize] );
 declare_value_conversion!(String => [SharedString] );
 declare_value_conversion!(Bool => [bool] );
-declare_value_conversion!(Image => [ImageReference] );
+declare_value_conversion!(Image => [Image] );
 declare_value_conversion!(Struct => [Struct] );
 declare_value_conversion!(Brush => [Brush] );
 declare_value_conversion!(PathElements => [PathData]);
 declare_value_conversion!(EasingCurve => [sixtyfps_corelib::animations::EasingCurve]);
+declare_value_conversion!(LayoutCache => [SharedVector<f32>] );
 
 /// Implement From / TryInto for Value that convert a `struct` to/from `Value::Object`
 macro_rules! declare_value_struct_conversion {
@@ -245,6 +255,7 @@ declare_value_struct_conversion!(struct sixtyfps_corelib::model::StandardListVie
 declare_value_struct_conversion!(struct sixtyfps_corelib::properties::StateInfo { current_state, previous_state, change_time });
 declare_value_struct_conversion!(struct sixtyfps_corelib::input::KeyboardModifiers { control, alt, shift, meta });
 declare_value_struct_conversion!(struct sixtyfps_corelib::input::KeyEvent { event_type, text, modifiers });
+declare_value_struct_conversion!(struct sixtyfps_corelib::layout::LayoutInfo { min, max, min_percent, max_percent, preferred, stretch });
 
 /// Implement From / TryInto for Value that convert an `enum` to/from `Value::EnumerationValue`
 ///
@@ -341,8 +352,8 @@ impl TryInto<sixtyfps_corelib::Color> for Value {
 /// This type represents a runtime instance of structure in `.60`.
 ///
 /// This can either be an instance of a name structure introduced
-/// with the `struct` keyword in the .60 file, or an annonymous struct
-/// writen with the `{ key: value, }`  notation.
+/// with the `struct` keyword in the .60 file, or an anonymous struct
+/// written with the `{ key: value, }`  notation.
 ///
 /// It can be constructed with the [`FromIterator`] trait, and converted
 /// into or from a [`Value`] with the [`From`] and [`TryInto`] trait
@@ -466,15 +477,15 @@ impl ComponentCompiler {
     ///
     /// Returns the compiled `ComponentDefinition` if there were no errors.
     ///
-    /// Any diagnostics produced during the compilation, such as warnigns or errors, are collected
+    /// Any diagnostics produced during the compilation, such as warnings or errors, are collected
     /// in this ComponentCompiler and can be retrieved after the call using the [[Self::diagnostics()]]
     /// function. The [`print_diagnostics`] function can be used to display the diagnostics
     /// to the users.
     ///
     /// Diagnostics from previous calls are cleared when calling this function.
     ///
-    /// This function is `async` but in practice, this is only asynchronious if
-    /// [`Self::set_file_loader`] was called and its future is actually asynchronious.
+    /// This function is `async` but in practice, this is only asynchronous if
+    /// [`Self::set_file_loader`] was called and its future is actually asynchronous.
     /// If that is not used, then it is fine to use a very simple executor, such as the one
     /// provided by the `spin_on` crate
     pub async fn build_from_path<P: AsRef<Path>>(
@@ -490,14 +501,11 @@ impl ComponentCompiler {
             }
         };
 
-        // We create here a 'static guard. That's alright because we make sure
-        // in this module that we only use erased component
-        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
-
+        generativity::make_guard!(guard);
         let (c, diag) =
             crate::dynamic_component::load(source, path.into(), self.config.clone(), guard).await;
         self.diagnostics = diag.into_iter().collect();
-        c.ok().map(|inner| ComponentDefinition { inner })
+        c.ok().map(|inner| ComponentDefinition { inner: inner.into() })
     }
 
     /// Compile some .60 code into a ComponentDefinition
@@ -512,8 +520,8 @@ impl ComponentCompiler {
     ///
     /// Diagnostics from previous calls are cleared when calling this function.
     ///
-    /// This function is `async` but in practice, this is only asynchronious if
-    /// [`Self::set_file_loader`] is set and its future is actually asynchronious.
+    /// This function is `async` but in practice, this is only asynchronous if
+    /// [`Self::set_file_loader`] is set and its future is actually asynchronous.
     /// If that is not used, then it is fine to use a very simple executor, such as the one
     /// provided by the `spin_on` crate
     pub async fn build_from_source(
@@ -521,14 +529,11 @@ impl ComponentCompiler {
         source_code: String,
         path: PathBuf,
     ) -> Option<ComponentDefinition> {
-        // We create here a 'static guard. That's alright because we make sure
-        // in this module that we only use erased component
-        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
-
+        generativity::make_guard!(guard);
         let (c, diag) =
             crate::dynamic_component::load(source_code, path, self.config.clone(), guard).await;
         self.diagnostics = diag.into_iter().collect();
-        c.ok().map(|inner| ComponentDefinition { inner })
+        c.ok().map(|inner| ComponentDefinition { inner: inner.into() })
     }
 }
 
@@ -541,14 +546,15 @@ impl ComponentCompiler {
 /// creating the instances it is safe to drop the ComponentDefinition.
 #[derive(Clone)]
 pub struct ComponentDefinition {
-    inner: Rc<crate::dynamic_component::ComponentDescription<'static>>,
+    inner: crate::dynamic_component::ErasedComponentDescription,
 }
 
 impl ComponentDefinition {
     /// Creates a new instance of the component and returns a shared handle to it.
     pub fn create(&self) -> ComponentInstance {
+        generativity::make_guard!(guard);
         ComponentInstance {
-            inner: self.inner.clone().create(
+            inner: self.inner.unerase(guard).clone().create(
                 #[cfg(target_arch = "wasm32")]
                 "canvas".into(),
             ),
@@ -558,17 +564,21 @@ impl ComponentDefinition {
     /// Instantiate the component for wasm using the given canvas id
     #[cfg(target_arch = "wasm32")]
     pub fn create_with_canvas_id(&self, canvas_id: &str) -> ComponentInstance {
-        ComponentInstance { inner: self.inner.clone().create(canvas_id.into()) }
+        generativity::make_guard!(guard);
+        ComponentInstance { inner: self.inner.unerase(guard).clone().create(canvas_id.into()) }
     }
 
-    /// Instentiate the component using an existing window.
+    /// Instantiate the component using an existing window.
     /// This method is internal because the Window is not a public type
     #[doc(hidden)]
     pub fn create_with_existing_window(
         &self,
         window: sixtyfps_corelib::window::ComponentWindow,
     ) -> ComponentInstance {
-        ComponentInstance { inner: self.inner.clone().create_with_existing_window(window) }
+        generativity::make_guard!(guard);
+        ComponentInstance {
+            inner: self.inner.unerase(guard).clone().create_with_existing_window(window),
+        }
     }
 
     /// List of publicly declared properties or callback.
@@ -578,12 +588,18 @@ impl ComponentDefinition {
     pub fn properties_and_callbacks(
         &self,
     ) -> impl Iterator<Item = (String, sixtyfps_compilerlib::langtype::Type)> + '_ {
-        self.inner.properties()
+        // We create here a 'static guard, because unfortunately the returned type would be restricted to the guard lifetime
+        // which is not required, but this is safe because there is only one instance of the unerased type
+        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
+        self.inner.unerase(guard).properties()
     }
 
     /// List of publicly declared properties.
     pub fn properties<'a>(&'a self) -> impl Iterator<Item = (String, ValueType)> + 'a {
-        self.inner.properties().filter_map(|(prop_name, prop_type)| {
+        // We create here a 'static guard, because unfortunately the returned type would be restricted to the guard lifetime
+        // which is not required, but this is safe because there is only one instance of the unerased type
+        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
+        self.inner.unerase(guard).properties().filter_map(|(prop_name, prop_type)| {
             if prop_type.is_property_type() {
                 Some((prop_name, prop_type.into()))
             } else {
@@ -592,9 +608,26 @@ impl ComponentDefinition {
         })
     }
 
+    /// Returns the names of all publicly declared callbacks.
+    pub fn callbacks<'a>(&'a self) -> impl Iterator<Item = String> + 'a {
+        // We create here a 'static guard, because unfortunately the returned type would be restricted to the guard lifetime
+        // which is not required, but this is safe because there is only one instance of the unerased type
+        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
+        self.inner.unerase(guard).properties().filter_map(|(prop_name, prop_type)| {
+            if matches!(prop_type, LangType::Callback { .. }) {
+                Some(prop_name)
+            } else {
+                None
+            }
+        })
+    }
+
     /// The name of this Component as written in the .60 file
     pub fn name(&self) -> &str {
-        self.inner.id()
+        // We create here a 'static guard, because unfortunately the returned type would be restricted to the guard lifetime
+        // which is not required, but this is safe because there is only one instance of the unerased type
+        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
+        self.inner.unerase(guard).id()
     }
 }
 
@@ -630,10 +663,8 @@ pub struct ComponentInstance {
 impl ComponentInstance {
     /// Return the [`ComponentDefinition`] that was used to create this instance.
     pub fn definition(&self) -> ComponentDefinition {
-        // We create here a 'static guard. That's alright because we make sure
-        // in this module that we only use erased component
-        let guard = unsafe { generativity::Guard::new(generativity::Id::new()) };
-        ComponentDefinition { inner: self.inner.unerase(guard).description() }
+        generativity::make_guard!(guard);
+        ComponentDefinition { inner: self.inner.unerase(guard).description().into() }
     }
 
     /// Return the value for a public property of this component.
@@ -756,7 +787,7 @@ impl ComponentInstance {
     /// Clone this `ComponentInstance`.
     ///
     /// A `ComponentInstance` is in fact a handle to a reference counted instance.
-    /// This function is semanticallt the same as the one from `Clone::clone`, but
+    /// This function is semantically the same as the one from `Clone::clone`, but
     /// Clone is not implemented because of the danger of circular reference:
     /// If you want to use this instance in a callback, you should capture a weak
     /// reference given by [`Self::as_weak`].
@@ -795,7 +826,7 @@ impl WeakComponentInstance {
         self.inner.upgrade().map(|inner| ComponentInstance { inner })
     }
 
-    /// Convenience function that returns a new stronlyg referenced component if
+    /// Convenience function that returns a new strongly referenced component if
     /// some other instance still holds a strong reference. Otherwise, this function
     /// panics.
     pub fn unwrap(&self) -> ComponentInstance {
@@ -846,7 +877,7 @@ pub fn run_event_loop() {
         .run_event_loop(sixtyfps_corelib::backend::EventLoopQuitBehavior::QuitOnLastWindowClosed);
 }
 
-/// This module constains a few function use by tests
+/// This module contains a few function use by tests
 pub mod testing {
     /// Wrapper around [`sixtyfps_corelib::tests::sixtyfps_send_mouse_click`]
     pub fn send_mouse_click(comp: &super::ComponentInstance, x: f32, y: f32) {
@@ -893,6 +924,34 @@ fn component_definition_properties() {
     assert_eq!(props[0].1, ValueType::String);
 }
 
+#[test]
+fn component_definition_properties2() {
+    let mut compiler = ComponentCompiler::new();
+    let comp_def = spin_on::spin_on(
+        compiler.build_from_source(
+            r#"
+    export Dummy := Rectangle {
+        property <string> sub-text <=> sub.text;
+        sub := Text { property <int> private-not-exported; }
+        callback hello;
+    }"#
+            .into(),
+            "".into(),
+        ),
+    )
+    .unwrap();
+
+    let props = comp_def.properties().collect::<Vec<(_, _)>>();
+
+    assert_eq!(props.len(), 1);
+    assert_eq!(props[0].0, "sub_text");
+    assert_eq!(props[0].1, ValueType::String);
+
+    let callbacks = comp_def.callbacks().collect::<Vec<_>>();
+    assert_eq!(callbacks.len(), 1);
+    assert_eq!(callbacks[0], "hello");
+}
+
 #[cfg(feature = "ffi")]
 #[allow(missing_docs)]
 #[path = "ffi.rs"]
