@@ -7,7 +7,7 @@
     This file is also available under commercial licensing terms.
     Please contact info@sixtyfps.io for more information.
 LICENSE END */
-use crate::api::Value;
+use crate::api::{Struct, Value};
 use crate::dynamic_component::InstanceRef;
 use core::convert::TryInto;
 use core::iter::FromIterator;
@@ -16,7 +16,7 @@ use corelib::graphics::{GradientStop, LinearGradientBrush, PathElement};
 use corelib::items::{ItemRef, PropertyAnimation};
 use corelib::rtti::AnimatedBindingKind;
 use corelib::window::ComponentWindow;
-use corelib::{Brush, Color, ImageReference, PathData, SharedString, SharedVector};
+use corelib::{Brush, Color, PathData, SharedString, SharedVector};
 use sixtyfps_compilerlib::expression_tree::{
     BindingExpression, BuiltinFunction, EasingCurve, Expression, Path as ExprPath,
     PathElement as ExprPathElement,
@@ -24,7 +24,7 @@ use sixtyfps_compilerlib::expression_tree::{
 use sixtyfps_compilerlib::langtype::Type;
 use sixtyfps_compilerlib::object_tree::ElementRc;
 use sixtyfps_corelib as corelib;
-use std::collections::HashMap;
+use std::collections::{BTreeMap, HashMap};
 use std::rc::Rc;
 
 pub trait ErasedPropertyInfo {
@@ -89,7 +89,7 @@ impl<Item: vtable::HasStaticVTable<corelib::items::ItemVTable>> ErasedCallbackIn
 impl corelib::rtti::ValueType for Value {}
 
 #[derive(Copy, Clone)]
-enum ComponentInstance<'a, 'id> {
+pub(crate) enum ComponentInstance<'a, 'id> {
     InstanceRef(InstanceRef<'a, 'id>),
     GlobalComponent(&'a Pin<Rc<dyn crate::global_component::GlobalComponent>>),
 }
@@ -98,7 +98,7 @@ enum ComponentInstance<'a, 'id> {
 pub struct EvalLocalContext<'a, 'id> {
     local_variables: HashMap<String, Value>,
     function_arguments: Vec<Value>,
-    component_instance: ComponentInstance<'a, 'id>,
+    pub(crate) component_instance: ComponentInstance<'a, 'id>,
     /// When Some, a return statement was executed and one must stop evaluating
     return_value: Option<Value>,
 }
@@ -125,15 +125,6 @@ impl<'a, 'id> EvalLocalContext<'a, 'id> {
             return_value: None,
         }
     }
-
-    pub fn from_global(global: &'a Pin<Rc<dyn crate::global_component::GlobalComponent>>) -> Self {
-        Self {
-            local_variables: Default::default(),
-            function_arguments: Default::default(),
-            component_instance: ComponentInstance::GlobalComponent(&global),
-            return_value: None,
-        }
-    }
 }
 
 /// Evaluate an expression and return a Value as the result of this expression
@@ -149,7 +140,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
         Expression::NumberLiteral(n, unit) => Value::Number(unit.normalize(*n)),
         Expression::BoolLiteral(b) => Value::Bool(*b),
         Expression::CallbackReference { .. } => panic!("callback in expression"),
-        Expression::BuiltinFunctionReference(_) => panic!(
+        Expression::BuiltinFunctionReference(..) => panic!(
             "naked builtin function reference not allowed, should be handled by function call"
         ),
         Expression::ElementReference(_) => todo!("Element references are only supported in the context of built-in function calls at the moment"),
@@ -202,87 +193,69 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
         }
         Expression::FunctionCall { function, arguments, source_location: _ } => match &**function {
             Expression::CallbackReference(nr) => {
-                let element = nr.element();
-                generativity::make_guard!(guard);
-                match enclosing_component_instance_for_element(&element, local_context.component_instance, guard) {
-                    ComponentInstance::InstanceRef(enclosing_component) => {
-                        let component_type = enclosing_component.component_type;
-                        let item_info = &component_type.items[element.borrow().id.as_str()];
-                        let item = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
-                        let args = arguments.iter().map(|e| eval_expression(e, local_context)).collect::<Vec<_>>();
-
-                        if let Some(callback) = item_info.rtti.callbacks.get(nr.name()) {
-                            callback.call(item, args.as_slice())
-                        } else if let Some(callback_offset) = component_type.custom_callbacks.get(nr.name())
-                        {
-                            let callback = callback_offset.apply(&*enclosing_component.instance);
-                            callback.call(args.as_slice())
-                        } else {
-                            panic!("unkown callback {}", nr.name())
-                        }
-                    }
-                    ComponentInstance::GlobalComponent(global) => {
-                        let args = arguments.iter().map(|e| eval_expression(e, local_context));
-                        global.as_ref().invoke_callback(nr.name(), args.collect::<Vec<_>>().as_slice())
-                    }
-                }
+                let args = arguments.iter().map(|e| eval_expression(e, local_context)).collect::<Vec<_>>();
+                invoke_callback(local_context.component_instance, &nr.element(), nr.name(), &args).unwrap()
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::GetWindowScaleFactor) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::GetWindowScaleFactor, _) => {
                 match local_context.component_instance {
                     ComponentInstance::InstanceRef(component) => Value::Number(window_ref(component).unwrap().scale_factor() as _),
                     ComponentInstance::GlobalComponent(_) => panic!("Cannot get the window from a global component"),
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Debug) => {
-                let a = arguments.iter().map(|e| eval_expression(e, local_context));
-                println!("{:?}", a.collect::<Vec<_>>());
+            Expression::BuiltinFunctionReference(BuiltinFunction::Debug, _) => {
+                let to_print: SharedString = eval_expression(&arguments[0], local_context).try_into().unwrap();
+                eprintln!("{}", to_print);
                 Value::Void
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Mod) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Mod, _) => {
                 let mut toint = |e| -> i32 { eval_expression(e, local_context).try_into().unwrap() };
                 Value::Number((toint(&arguments[0]) % toint(&arguments[1])) as _)
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Round) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Round, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.round())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Ceil) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Ceil, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.ceil())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Floor) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Floor, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.floor())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Sqrt) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Sqrt, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.sqrt())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Sin) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Abs, _) => {
+                let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
+                Value::Number(x.abs())
+            }
+            Expression::BuiltinFunctionReference(BuiltinFunction::Sin, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.to_radians().sin())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Cos) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Cos, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.to_radians().cos())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Tan) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::Tan, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.to_radians().tan())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ASin) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ASin, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.asin().to_degrees())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ACos) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ACos, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.acos().to_degrees())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ATan) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ATan, _) => {
                 let x: f64 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 Value::Number(x.atan().to_degrees())
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::SetFocusItem) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::SetFocusItem, _) => {
                 if arguments.len() != 1 {
                     panic!("internal error: incorrect argument count to SetFocusItem")
                 }
@@ -308,7 +281,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("internal error: argument to SetFocusItem must be an element")
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ShowPopupWindow) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ShowPopupWindow, _) => {
                 if arguments.len() != 1 {
                     panic!("internal error: incorrect argument count to ShowPopupWindow")
                 }
@@ -330,7 +303,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("internal error: argument to SetFocusItem must be an element")
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::StringIsFloat) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::StringIsFloat, _) => {
                 if arguments.len() != 1 {
                     panic!("internal error: incorrect argument count to StringIsFloat")
                 }
@@ -340,7 +313,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("Argument not a string");
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::StringToFloat) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::StringToFloat, _) => {
                 if arguments.len() != 1 {
                     panic!("internal error: incorrect argument count to StringToFloat")
                 }
@@ -350,7 +323,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("Argument not a string");
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ColorBrighter) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ColorBrighter, _) => {
                 if arguments.len() != 2 {
                     panic!("internal error: incorrect argument count to ColorBrighter")
                 }
@@ -364,7 +337,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("First argument not a color");
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ColorDarker) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ColorDarker, _) => {
                 if arguments.len() != 2 {
                     panic!("internal error: incorrect argument count to ColorDarker")
                 }
@@ -378,7 +351,22 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     panic!("First argument not a color");
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::Rgb) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::ImageSize, _) => {
+                if arguments.len() != 1 {
+                    panic!("internal error: incorrect argument count to ColorDarker")
+                }
+                if let Value::Image(img) = eval_expression(&arguments[0], local_context) {
+                    let size = img.size();
+                    let values = std::array::IntoIter::new([
+                        ("width".to_string(), Value::Number(size.width as f64)),
+                        ("height".to_string(), Value::Number(size.height as f64)),
+                    ]).collect();
+                    Value::Struct(values)
+                } else {
+                    panic!("First argument not an image");
+                }
+            }
+            Expression::BuiltinFunctionReference(BuiltinFunction::Rgb, _) => {
                 let r: i32 = eval_expression(&arguments[0], local_context).try_into().unwrap();
                 let g: i32 = eval_expression(&arguments[1], local_context).try_into().unwrap();
                 let b: i32 = eval_expression(&arguments[2], local_context).try_into().unwrap();
@@ -389,15 +377,12 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                 let a: u8 = (255. * a).max(0.).min(255.) as u8;
                 Value::Brush(Brush::SolidColor(Color::from_argb_u8(a, r, g, b)))
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::ImplicitItemSize) => {
-                if arguments.len() != 1 {
-                    panic!("internal error: incorrect argument count to ImplicitItemSize")
-                }
+            Expression::BuiltinFunctionReference(BuiltinFunction::ImplicitLayoutInfo(orient), _) => {
                 let component = match  local_context.component_instance  {
                     ComponentInstance::InstanceRef(c) => c,
                     ComponentInstance::GlobalComponent(_) => panic!("Cannot access the implicit item size from a global component")
                 };
-                if let Expression::ElementReference(item) = &arguments[0] {
+                if let [Expression::ElementReference(item)] = arguments.as_slice() {
                     generativity::make_guard!(guard);
 
                     let item = item.upgrade().unwrap();
@@ -408,26 +393,18 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
                     let item_ref = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
 
                     let window = window_ref(component).unwrap();
-
-                    let size = item_ref.as_ref().implicit_size(&window);
-                    let values = [
-                        ("width".to_string(), Value::Number(size.width as f64)),
-                        ("height".to_string(), Value::Number(size.height as f64)),
-                    ]
-                    .iter()
-                    .map(|(name, value)| (name.clone(), value.clone())).collect();
-                    Value::Struct(values)
+                    item_ref.as_ref().layouting_info(crate::eval_layout::to_runtime(*orient), &window).into()
                 } else {
-                    panic!("internal error: argument to ImplicitItemWidth must be an element")
+                    panic!("internal error: incorrect arguments to ImplicitLayoutInfo {:?}", arguments);
                 }
             }
-            Expression::BuiltinFunctionReference(BuiltinFunction::RegisterCustomFontByPath) => {
+            Expression::BuiltinFunctionReference(BuiltinFunction::RegisterCustomFontByPath, _) => {
                 if arguments.len() != 1 {
                     panic!("internal error: incorrect argument count to RegisterCustomFontByPath")
                 }
                 if let Value::String(s) = eval_expression(&arguments[0], local_context) {
                     if let Some(err) = crate::register_font_from_path(&std::path::PathBuf::from(s.as_str())).err() {
-                        sixtyfps_corelib::debug_log!("Error loading custom font {}: {}", s.as_str(), err);
+                        corelib::debug_log!("Error loading custom font {}: {}", s.as_str(), err);
                     }
                     Value::Void
                 } else {
@@ -438,7 +415,7 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
         }
         Expression::SelfAssignment { lhs, rhs, op } => {
             let rhs = eval_expression(&**rhs, local_context);
-            eval_assignement(lhs, *op, rhs, local_context);
+            eval_assignment(lhs, *op, rhs, local_context);
             Value::Void
         }
         Expression::BinaryExpression { lhs, rhs, op } => {
@@ -448,6 +425,15 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
             match (op, lhs, rhs) {
                 ('+', Value::String(mut a), Value::String(b)) => { a.push_str(b.as_str()); Value::String(a) },
                 ('+', Value::Number(a), Value::Number(b)) => Value::Number(a + b),
+                ('+', a @ Value::Struct(_), b @ Value::Struct(_)) => {
+                    let a : Option<corelib::layout::LayoutInfo> = a.try_into().ok();
+                    let b : Option<corelib::layout::LayoutInfo> = b.try_into().ok();
+                    if let (Some(a), Some(b)) = (a, b) {
+                        a.merge(&b).into()
+                    } else {
+                        panic!("unsupported {:?} {} {:?}", a, op, b);
+                    }
+                }
                 ('-', Value::Number(a), Value::Number(b)) => Value::Number(a - b),
                 ('/', Value::Number(a), Value::Number(b)) => Value::Number(a / b),
                 ('*', Value::Number(a), Value::Number(b)) => Value::Number(a * b),
@@ -476,15 +462,18 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
             }
         }
         Expression::ImageReference(resource_ref) => {
-            match resource_ref {
+            Value::Image(match resource_ref {
                 sixtyfps_compilerlib::expression_tree::ImageReference::None => {
-                    Value::Image(ImageReference::None)
+                    Ok(Default::default())
                 }
                 sixtyfps_compilerlib::expression_tree::ImageReference::AbsolutePath(path) => {
-                    Value::Image(ImageReference::AbsoluteFilePath(path.into()))
+                    corelib::graphics::Image::load_from_path(std::path::Path::new(path))
                 }
                 sixtyfps_compilerlib::expression_tree::ImageReference::EmbeddedData(_) => panic!("Resource embedding is not supported by the interpreter")
-            }
+            }.unwrap_or_else(|_| {
+                eprintln!("Could not load image {:?}",resource_ref );
+                Default::default()
+            }))
         }
         Expression::Condition { condition, true_expr, false_expr } => {
             match eval_expression(&**condition, local_context).try_into()
@@ -539,10 +528,25 @@ pub fn eval_expression(e: &Expression, local_context: &mut EvalLocalContext) ->
             }
             local_context.return_value.clone().unwrap()
         }
+        Expression::LayoutCacheAccess { layout_cache_prop, index, repeater_index } => {
+            let cache = load_property_helper(local_context.component_instance, &layout_cache_prop.element(), layout_cache_prop.name()).unwrap();
+            if let Value::LayoutCache(cache) = cache {
+                if let Some(ri) = repeater_index {
+                    let offset : usize = eval_expression(&ri, local_context).try_into().unwrap();
+                    Value::Number(cache[(cache[*index] as usize) + offset * 2].into())
+                } else {
+                    Value::Number(cache[*index].into())
+                }
+            } else {
+                panic!("invalid layout cache")
+            }
+        }
+        Expression::ComputeLayoutInfo(lay, o) => crate::eval_layout::compute_layout_info(lay, *o, local_context),
+        Expression::SolveLayout(lay, o) => crate::eval_layout::solve_layout(lay, *o, local_context),
     }
 }
 
-fn eval_assignement(lhs: &Expression, op: char, rhs: Value, local_context: &mut EvalLocalContext) {
+fn eval_assignment(lhs: &Expression, op: char, rhs: Value, local_context: &mut EvalLocalContext) {
     let eval = |lhs| match (lhs, &rhs, op) {
         (Value::String(ref mut a), Value::String(b), '+') => {
             a.push_str(b.as_str());
@@ -604,7 +608,7 @@ fn eval_assignement(lhs: &Expression, op: char, rhs: Value, local_context: &mut
                 let mut r = o.get_field(name).unwrap().clone();
                 r = if op == '=' { rhs } else { eval(std::mem::take(&mut r)) };
                 o.set_field(name.to_owned(), r);
-                eval_assignement(base, '=', Value::Struct(o), local_context)
+                eval_assignment(base, '=', Value::Struct(o), local_context)
             }
         }
         Expression::RepeaterModelReference { element } => {
@@ -671,7 +675,7 @@ fn load_property_helper(
                 .component_type
                 .items
                 .get(element.id.as_str())
-                .unwrap_or_else(|| panic!("Unkown element for {}.{}", element.id, name));
+                .unwrap_or_else(|| panic!("Unknown element for {}.{}", element.id, name));
             core::mem::drop(element);
             let item = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
             Ok(item_info.rtti.properties.get(name).ok_or(())?.get(item))
@@ -710,6 +714,35 @@ pub fn store_property(
     Ok(())
 }
 
+pub(crate) fn invoke_callback(
+    component_instance: ComponentInstance,
+    element: &ElementRc,
+    callback_name: &str,
+    args: &[Value],
+) -> Option<Value> {
+    generativity::make_guard!(guard);
+    match enclosing_component_instance_for_element(element, component_instance, guard) {
+        ComponentInstance::InstanceRef(enclosing_component) => {
+            let component_type = enclosing_component.component_type;
+            let item_info = &component_type.items[element.borrow().id.as_str()];
+            let item = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
+
+            if let Some(callback) = item_info.rtti.callbacks.get(callback_name) {
+                Some(callback.call(item, args))
+            } else if let Some(callback_offset) = component_type.custom_callbacks.get(callback_name)
+            {
+                let callback = callback_offset.apply(&*enclosing_component.instance);
+                Some(callback.call(args))
+            } else {
+                None
+            }
+        }
+        ComponentInstance::GlobalComponent(global) => {
+            Some(global.as_ref().invoke_callback(callback_name, args))
+        }
+    }
+}
+
 fn root_component_instance<'a, 'old_id, 'new_id>(
     component: InstanceRef<'a, 'old_id>,
     guard: generativity::Guard<'new_id>,
@@ -748,13 +781,13 @@ pub fn enclosing_component_for_element<'a, 'old_id, 'new_id>(
     guard: generativity::Guard<'new_id>,
 ) -> InstanceRef<'a, 'new_id> {
     let enclosing = &element.borrow().enclosing_component.upgrade().unwrap();
-    assert!(!enclosing.is_global());
     if Rc::ptr_eq(enclosing, &component.component_type.original) {
         // Safety: new_id is an unique id
         unsafe {
             std::mem::transmute::<InstanceRef<'a, 'old_id>, InstanceRef<'a, 'new_id>>(component)
         }
     } else {
+        assert!(!enclosing.is_global());
         let parent_component = component
             .component_type
             .parent_component_offset
@@ -771,8 +804,8 @@ pub fn enclosing_component_for_element<'a, 'old_id, 'new_id>(
 }
 
 /// Return the component instance which hold the given element.
-/// The difference with enclosing_component_for_element is that it taked in account the GlobalComponent.
-fn enclosing_component_instance_for_element<'a, 'old_id, 'new_id>(
+/// The difference with enclosing_component_for_element is that it takes the GlobalComponent into account.
+pub(crate) fn enclosing_component_instance_for_element<'a, 'old_id, 'new_id>(
     element: &'a ElementRc,
     component_instance: ComponentInstance<'a, 'old_id>,
     guard: generativity::Guard<'new_id>,
@@ -780,7 +813,7 @@ fn enclosing_component_instance_for_element<'a, 'old_id, 'new_id>(
     let enclosing = &element.borrow().enclosing_component.upgrade().unwrap();
     match component_instance {
         ComponentInstance::InstanceRef(component) => {
-            if enclosing.is_global() {
+            if enclosing.is_global() && !component.component_type.original.is_global() {
                 // we need a 'static guard in order to be able to borrow from `root` otherwise it does not work because of variance.
                 // Safety: This is the only 'static Id in scope.
                 let static_guard =
@@ -803,10 +836,8 @@ fn enclosing_component_instance_for_element<'a, 'old_id, 'new_id>(
     }
 }
 
-pub fn new_struct_with_bindings<
-    ElementType: 'static + Default + sixtyfps_corelib::rtti::BuiltinItem,
->(
-    bindings: &HashMap<String, BindingExpression>,
+pub fn new_struct_with_bindings<ElementType: 'static + Default + corelib::rtti::BuiltinItem>(
+    bindings: &BTreeMap<String, BindingExpression>,
     local_context: &mut EvalLocalContext,
 ) -> ElementType {
     let mut element = ElementType::default();
@@ -822,8 +853,8 @@ pub fn new_struct_with_bindings<
 fn convert_from_lyon_path<'a>(
     it: impl IntoIterator<Item = &'a lyon_path::Event<lyon_path::math::Point, lyon_path::math::Point>>,
 ) -> PathData {
+    use corelib::graphics::PathEvent;
     use lyon_path::Event;
-    use sixtyfps_corelib::graphics::PathEvent;
 
     let mut coordinates = Vec::new();
 
@@ -905,3 +936,41 @@ fn convert_path_element(
         ),
     }
 }
+
+/// Create a value suitable as the default value of a given type
+pub fn default_value_for_type(ty: &Type) -> Value {
+    match ty {
+        Type::Float32 | Type::Int32 => Value::Number(0.),
+        Type::String => Value::String(Default::default()),
+        Type::Color | Type::Brush => Value::Brush(Default::default()),
+        Type::Duration | Type::Angle | Type::PhysicalLength | Type::LogicalLength => {
+            Value::Number(0.)
+        }
+        Type::Image => Value::Image(Default::default()),
+        Type::Bool => Value::Bool(false),
+        Type::Callback { .. } => Value::Void,
+        Type::Struct { fields, .. } => Value::Struct(Struct::from_iter(
+            fields.iter().map(|(n, t)| (n.clone(), default_value_for_type(t))),
+        )),
+        Type::Array(_) => Value::Array(Default::default()),
+        Type::Percent => Value::Number(0.),
+        Type::Enumeration(e) => {
+            Value::EnumerationValue(e.name.clone(), e.values.get(e.default_value).unwrap().clone())
+        }
+        Type::Easing => Value::EasingCurve(Default::default()),
+        Type::Void | Type::Invalid => Value::Void,
+        Type::Model => Value::Void,
+        Type::UnitProduct(_) => Value::Number(0.),
+        Type::PathElements => Value::PathElements(Default::default()),
+        Type::LayoutCache => Value::LayoutCache(Default::default()),
+        Type::InferredProperty
+        | Type::InferredCallback
+        | Type::ElementReference
+        | Type::Builtin(_)
+        | Type::Component(_)
+        | Type::Native(_)
+        | Type::Function { .. } => {
+            panic!("There can't be such property")
+        }
+    }
+}
