@@ -291,7 +291,7 @@ pub extern "C" fn sixtyfps_interpreter_struct_make_iter<'a>(
 }
 
 /// Get a property.
-/// The `out` parameter must be uninitiaized. If this function returns true, the out will be initialized
+/// The `out` parameter must be uninitialized. If this function returns true, the out will be initialized
 /// to the resulting value. If this function returns false, out is unchanged
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_interpreter_component_instance_get_property(
@@ -324,7 +324,7 @@ pub extern "C" fn sixtyfps_interpreter_component_instance_set_property(
 }
 
 /// Invoke a callback.
-/// The `out` parameter must be uninitiaized. If this function returns true, the out will be initialized
+/// The `out` parameter must be uninitialized. If this function returns true, the out will be initialized
 /// to the resulting value. If this function returns false, out is unchanged
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_interpreter_component_instance_invoke_callback(
@@ -526,6 +526,7 @@ pub unsafe extern "C" fn sixtyfps_interpreter_model_notify_row_removed(
 }
 
 // FIXME: Figure out how to re-export the one from compilerlib
+// Note: Documented in sixtyfps.h - keep in sync!
 #[derive(Clone)]
 #[repr(C)]
 pub enum CDiagnosticLevel {
@@ -533,6 +534,7 @@ pub enum CDiagnosticLevel {
     Warning,
 }
 
+// Note: Documented in sixtyfps.h - keep in sync!
 #[derive(Clone)]
 #[repr(C)]
 pub struct CDiagnostic {
@@ -679,6 +681,7 @@ pub unsafe extern "C" fn sixtyfps_interpreter_component_compiler_build_from_path
     }
 }
 
+// NOTE: The C++ documentation for this exists in sixtyfps_interpreter.h. Keep in sync!
 #[derive(Clone)]
 #[repr(C)]
 pub struct PropertyDescriptor {
@@ -712,7 +715,7 @@ pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_clone(
     std::ptr::write(def as *mut ComponentDefinition, other.as_component_definition().clone())
 }
 
-/// Destruct the value in that memory location
+/// Destruct the component definition in that memory location
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_destructor(
     val: *mut ComponentDefinitionOpaque,
@@ -720,7 +723,7 @@ pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_destructor(
     drop(std::ptr::read(val as *mut ComponentDefinition))
 }
 
-/// Construct a new Value in the given memory location
+/// Returns the list of properties of the component the component definition describes
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_properties(
     def: &ComponentDefinitionOpaque,
@@ -733,3 +736,21 @@ pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_properties(
         },
     ))
 }
+
+/// Returns the list of callback names of the component the component definition describes
+#[no_mangle]
+pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_callbacks(
+    def: &ComponentDefinitionOpaque,
+    callbacks: &mut SharedVector<SharedString>,
+) {
+    callbacks.extend((&*def).as_component_definition().callbacks().map(|name| name.into()))
+}
+
+/// Return the name of the component definition
+#[no_mangle]
+pub unsafe extern "C" fn sixtyfps_interpreter_component_definition_name(
+    def: &ComponentDefinitionOpaque,
+    name: &mut SharedString,
+) {
+    *name = (&*def).as_component_definition().name().into()
+}
